@@ -2,7 +2,7 @@ package Mojo::JWT;
 
 use Mojo::Base -base;
 
-our $VERSION = '0.04';
+our $VERSION = '0.06';
 $VERSION = eval $VERSION;
 
 use Mojo::JSON qw/encode_json decode_json/;
@@ -10,6 +10,7 @@ use MIME::Base64 qw/encode_base64url decode_base64url/;
 
 use Carp;
 
+has header => sub { {} };
 has algorithm => 'HS256';
 has [qw/allow_none set_iat/] => 0;
 has claims => sub { {} };
@@ -25,16 +26,18 @@ sub decode {
 
   # reset
   $self->algorithm(undef);
-  delete $self->{$_} for qw/claims expires not_before/;
+  delete $self->{$_} for qw/claims expires not_before header/;
 
   my ($hstring, $cstring, $signature) = split /\./, $token;
   my $header = decode_json decode_base64url($hstring);
   my $claims = decode_json decode_base64url($cstring);
   $signature = decode_base64url $signature;
 
-  #croak 'Not a JWT' unless $header->{typ} eq 'JWT';
+  croak 'Missing JWT "typ" header' unless defined $header->{typ};
+  croak 'Not a JWT' unless delete $header->{typ} eq 'JWT';
   croak 'Required header field "alg" not specified'
-    unless my $algo = $self->algorithm($header->{alg})->algorithm;
+    unless my $algo = $self->algorithm(delete $header->{alg})->algorithm;
+  $self->header($header);
 
   $self->$peek($claims) if $peek;
 
@@ -76,7 +79,8 @@ sub encode {
   if (defined(my $exp = $self->expires))    { $claims->{exp} = $exp }
   if (defined(my $nbf = $self->not_before)) { $claims->{nbf} = $nbf }
 
-  my $hstring = encode_base64url encode_json($self->header);
+  my $header  = { %{ $self->header }, typ => 'JWT', alg => $self->algorithm };
+  my $hstring = encode_base64url encode_json($header);
   my $cstring = encode_base64url encode_json($claims);
   my $payload = "$hstring.$cstring";
   my $signature;
@@ -94,8 +98,6 @@ sub encode {
   return $self->{token} = "$payload." . encode_base64url $signature;
 }
 
-sub header { { typ => 'JWT', alg => shift->algorithm } }
-
 sub now { time }
 
 sub sign_hmac {
@@ -138,7 +140,7 @@ Mojo::JWT - JSON Web Token the Mojo way
 
 =head1 DESCRIPTION
 
-JSON Web Token is described in L<https://tools.ietf.org/html/draft-ietf-oauth-json-web-token-32>.
+JSON Web Token is described in L<https://tools.ietf.org/html/rfc7519>.
 L<Mojo::JWT> implements that standard with an API that should feel familiar to L<Mojolicious> users (though of course it is useful elsewhere).
 Indeed, JWT is much like L<Mojolicious::Sessions> except that the result is a url-safe text string rather than a cookie.
 
@@ -172,6 +174,10 @@ This must be a hash reference, array references are not allowed as the top-level
 The epoch time value after which the JWT value should not be considered valid.
 This value (if set and not undefined) will be used as the C<exp> key in the claims or was extracted from the claims during the most recent decoding.
 
+=head2 header
+
+You may set your own headers when encoding the JWT bypassing a hash reference to the L</header> attribute. Please note that there are two default headers set. B<alg> is set to the value of L</algorithm> or 'HS256' and B<typ> is set to 'JWT'. These cannot be overridden.
+
 =head2 not_before
 
 The epoch time value before which the JWT value should not be considered valid.
@@ -306,17 +312,17 @@ This method is provided mostly for the purposes of subclassing.
 
 L<http://github.com/jberger/Mojo-JWT>
 
-=head1 DEVELOPMENT SPONSORED BY
-
-Restore Health Corporation, L<http://restorehc.com>
-
 =head1 AUTHOR
 
 Joel Berger, E<lt>joel.a.berger@gmail.comE<gt>
 
+=head1 CONTRIBUTORS
+
+Christopher Raa (mishanti1)
+
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (C) 2015 by Joel Berger
+Copyright (C) 2015 by L</AUTHOR> and L</CONTRIBTORS>.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
